@@ -692,7 +692,10 @@ setMethod("project", signature(x="SpatRaster"),
 			}
 		} else {
 			if (!is.character(y)) {
-				warn("project,SpatRaster", "argument y (the crs) should be a character value")
+				#warn("project,SpatRaster", "argument y (the crs) should be a character value")
+				if (inherits(y, "numeric")) {
+					error("project,SpatRaster", "argument y (the crs) cannot be a number.\nFor EPSG codes use this format 'epsg:1234'")				
+				}
 				y <- as.character(crs(y))
 			}
 			if (!is.null(res) || !is.null(origin)) {
@@ -881,7 +884,7 @@ setMethod("rotate", signature(x="SpatRaster"),
 )
 
 setMethod("rotate", signature(x="SpatVector"),
-	function(x, longitude=0, split=FALSE, left=TRUE, normalize=FALSE) {
+	function(x, longitude=0, split=TRUE, left=TRUE, normalize=FALSE) {
 		if (split) {
 			e <- ext(x)
 			if ((longitude < e$xmin) || (longitude > e$xmax)) {
@@ -895,6 +898,8 @@ setMethod("rotate", signature(x="SpatVector"),
 			ew <- ext(c(e[1], longitude, e[3:4]))
 			ee <- ext(c(longitude, e[2:4]))
 			x$unique_id_for_aggregation <- 1:nrow(x)
+			xcrs <- crs(x)
+			crs(x) <- NULL  # avoid wrapping
 			xw <- crop(x, ew)
 			xe <- crop(x, ee)
 			if (left) {
@@ -910,6 +915,7 @@ setMethod("rotate", signature(x="SpatVector"),
 				x <- out
 			} 
 			x$unique_id_for_aggregation <- NULL
+			crs(x) <- xcrs
 		} else {
 			x@pntr <- x@pntr$rotate_longitude(longitude, left)
 			x <- messages(x, "rotate")
@@ -1158,7 +1164,11 @@ setMethod("unique", signature(x="SpatRaster", incomparables="ANY"),
 
 		opt <- spatOptions()
 
-		if (as.raster) incomparables = FALSE
+		if (as.raster) {
+			if (nlyr(x) == 1) return(x)
+			incomparables = FALSE
+		}
+		
 		u <- x@pntr$unique(incomparables, digits, na.rm[1], opt)
 
 		if (!as.raster) {
