@@ -9,6 +9,7 @@
 //static void SpatRaster_finalizer( SpatRaster* ptr ){
 //}
 
+
 /*
 Rcpp::List getBlockSizeR(SpatRaster* r, unsigned n, double frac) {
 	SpatOptions opt;
@@ -665,7 +666,7 @@ RCPP_MODULE(spat){
 	class_<SpatRaster>("SpatRaster")
 		.constructor()
 	 // .constructor<std::string, int>()
-		.constructor<std::vector<std::string>, std::vector<int>, std::vector<std::string>, bool, std::vector<std::string>, std::vector<std::string>, std::vector<size_t>, bool, std::vector<std::string>>()
+		.constructor<std::vector<std::string>, std::vector<int>, std::vector<std::string>, bool, std::vector<std::string>, std::vector<std::string>, std::vector<size_t>, bool, bool, std::vector<std::string>>()
 		
 		.constructor<std::vector<size_t>, std::vector<double>, std::string>()
 		//.finalizer(&SpatRaster_finalizer)
@@ -1043,7 +1044,7 @@ RCPP_MODULE(spat){
 
 	class_<SpatRasterCollection>("SpatRasterCollection")
 		.constructor()
-		.constructor<std::string, std::vector<int>, bool, std::vector<std::string>, std::vector<std::string>>()
+		.constructor<std::string, std::vector<int>, bool, std::vector<std::string>, bool, bool, std::vector<std::string>>()
 
 		.property("names", &SpatRasterCollection::get_names, &SpatRasterCollection::set_names)
 
@@ -1072,7 +1073,7 @@ RCPP_MODULE(spat){
 
 	class_<SpatRasterStack>("SpatRasterStack")
 		.constructor()
-		.constructor<std::string, std::vector<int>, bool, std::vector<std::string>, std::vector<std::string>>()
+		.constructor<std::string, std::vector<int>, bool, std::vector<std::string>, bool, bool, std::vector<std::string>>()
 		.constructor<SpatRaster, std::string, std::string, std::string>()
 		.method("deepcopy", &SpatRasterStack::deepCopy)
 
