@@ -310,8 +310,10 @@ bool SpatRaster::readAll() {
 		if (!source[src].memory) {
 			readChunkGDAL(source[src].values, src, row, nrows, col, ncols);
 			source[src].memory = true;
+			source[src].extset = false;
+			source[src].flipped = false;
 			source[src].filename = "";
-			std::iota(source[src].layers.begin(), source[src].layers.end(), 0);
+			std::iota(source[src].layers.begin(), source[src].layers.end(), 0);			
 		}
 		if (src > 0) {
 			if (!source[0].combine_sources(source[src])) {
